/*
 * Copyright (C) 2000-2001 Damien Douxchamps  <douxchamps@ieee.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "callbacks.h"
#include "interface.h"
#include "update.h"
#include "support.h"
#include "activate.h"
#include "callback_proc.h"
#include "definitions.h"
#include <libdc1394/dc1394_control.h>
#include <libraw1394/raw1394.h>
#include "video1394.h"


extern GtkWidget *commander_window;
extern GtkWidget *porthole_window;
extern GtkWidget *color_window;
extern GtkWidget *aperture_window;
extern GtkWidget *status_window;
extern GtkWidget *about_window;
extern GtkWidget *capture_window;
extern GtkWidget *temperature_window;
extern GtkWidget *format7_window;
extern dc1394_camerainfo *camera;
extern dc1394_feature_set *feature_set;
extern dc1394_camerainfo *cameras;
extern dc1394_feature_set *feature_sets;
extern dc1394_miscinfo *misc_info;
extern dc1394_miscinfo *misc_infos;
extern int current_camera;

gboolean
on_commander_window_delete_event       (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_exit(0);
  return FALSE;
}


void
on_file_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left blank: intermediate menu
}


void
on_load_setup_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *dialog = create_file_selector_window();
  gtk_window_set_title (GTK_WINDOW (dialog), "Select a setup file to load...");
  gtk_widget_show(dialog);

  // to be continued...
}


void
on_save_setup_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // TO BE FILLED
}


void
on_save_setup_as_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *dialog = create_file_selector_window();
  gtk_window_set_title (GTK_WINDOW (dialog), "Save setup as...");
  gtk_widget_show(dialog);

  // to be continued...
}


void
on_exit_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_exit(0);
}


void
on_porthole_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (porthole_window);
}


void
on_color_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (color_window);
}


void
on_aperture_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (aperture_window);
}


void
on_format_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left blank: intermediate menu
}


void
on_format_0_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left blank: intermediate menu
}



void
on_format_1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left blank: intermediate menu
}

void
on_format_2_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left blank: intermediate menu
}


void
on_format_6_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left blank: intermediate menu
}


void
on_format_7_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left blank: intermediate menu
}

void
on_help_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left blank: intermediate menu
}


void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  about_window = create_about_window ();
  gtk_widget_show (about_window);
}


gboolean
on_file_selector_window_delete_event   (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  GtkWidget *dialog = gtk_widget_get_toplevel(widget);
  gtk_widget_destroy(dialog);
  return FALSE;
}

void
on_f0_mode0_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,0);
  err=dc1394_set_video_mode(camera->handle,camera->id,0);
  misc_info->format=0;
  misc_info->mode=0;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f0_mode1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,0);
  err=dc1394_set_video_mode(camera->handle,camera->id,1);
  misc_info->format=0;
  misc_info->mode=1;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f0_mode2_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,0);
  err=dc1394_set_video_mode(camera->handle,camera->id,2);
  misc_info->format=0;
  misc_info->mode=2;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f0_mode3_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,0);
  err=dc1394_set_video_mode(camera->handle,camera->id,3);
  misc_info->format=0;
  misc_info->mode=3;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f0_mode4_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,0);
  err=dc1394_set_video_mode(camera->handle,camera->id,4);
  misc_info->format=0;
  misc_info->mode=4;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f0_mode5_ativate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,0);
  err=dc1394_set_video_mode(camera->handle,camera->id,5);
  misc_info->format=0;
  misc_info->mode=5;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f1_mode0_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,1);
  err=dc1394_set_video_mode(camera->handle,camera->id,0);
  misc_info->format=1;
  misc_info->mode=0;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f1_mode1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,1);
  err=dc1394_set_video_mode(camera->handle,camera->id,1);
  misc_info->format=1;
  misc_info->mode=1;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f1_mode2_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,1);
  err=dc1394_set_video_mode(camera->handle,camera->id,2);
  misc_info->format=1;
  misc_info->mode=2;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f1_mode3_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,1);
  err=dc1394_set_video_mode(camera->handle,camera->id,3);
  misc_info->format=1;
  misc_info->mode=3;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f1_mode4_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,1);
  err=dc1394_set_video_mode(camera->handle,camera->id,4);
  misc_info->format=1;
  misc_info->mode=4;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f1_mode5_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,1);
  err=dc1394_set_video_mode(camera->handle,camera->id,5);
  misc_info->format=1;
  misc_info->mode=5;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f2_mode0_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,2);
  err=dc1394_set_video_mode(camera->handle,camera->id,0);
  misc_info->format=2;
  misc_info->mode=0;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f2_mode1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,2);
  err=dc1394_set_video_mode(camera->handle,camera->id,1);
  misc_info->format=2;
  misc_info->mode=1;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f2_mode2_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,2);
  err=dc1394_set_video_mode(camera->handle,camera->id,2);
  misc_info->format=2;
  misc_info->mode=2;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f2_mode3_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,2);
  err=dc1394_set_video_mode(camera->handle,camera->id,3);
  misc_info->format=2;
  misc_info->mode=3;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f2_mode4_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,2);
  err=dc1394_set_video_mode(camera->handle,camera->id,4);
  misc_info->format=2;
  misc_info->mode=4;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f2_mode5_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,2);
  err=dc1394_set_video_mode(camera->handle,camera->id,5);
  misc_info->format=2;
  misc_info->mode=5;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f6_mode0_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,6);
  err=dc1394_set_video_mode(camera->handle,camera->id,0);
  misc_info->format=6;
  misc_info->mode=0;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f7_mode0_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,7);
  err=dc1394_set_video_mode(camera->handle,camera->id,0);
  misc_info->format=7;
  misc_info->mode=0;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f7_mode1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,7);
  err=dc1394_set_video_mode(camera->handle,camera->id,1);
  misc_info->format=7;
  misc_info->mode=1;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f7_mode2_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,7);
  err=dc1394_set_video_mode(camera->handle,camera->id,2);
  misc_info->format=7;
  misc_info->mode=2;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f7_mode3_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,7);
  err=dc1394_set_video_mode(camera->handle,camera->id,3);
  misc_info->format=7;
  misc_info->mode=3;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f7_mode4_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,7);
  err=dc1394_set_video_mode(camera->handle,camera->id,4);
  misc_info->format=7;
  misc_info->mode=4;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f7_mode5_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,7);
  err=dc1394_set_video_mode(camera->handle,camera->id,5);
  misc_info->format=7;
  misc_info->mode=5;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f7_mode6_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,7);
  err=dc1394_set_video_mode(camera->handle,camera->id,6);
  misc_info->format=7;
  misc_info->mode=6;
  UpdateFpsMenu();
  IsoFlowResume();
}


void
on_f7_mode7_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,7);
  err=dc1394_set_video_mode(camera->handle,camera->id,7);
  misc_info->format=7;
  misc_info->mode=7;
  UpdateFpsMenu();
  IsoFlowResume();
}

void
on_fps_menu_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left blank: intermediate menu
}


void
on_fps_activate                    (GtkMenuItem     *menuitem,
				    gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_framerate(camera->handle, camera->id, (int)user_data);
  misc_info->framerate=(int)user_data;
  IsoFlowResume();
}


void
on_power_on_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  int err;
  err=dc1394_camera_on(camera->handle, camera->id);
}


void
on_power_off_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  int err;
  err=dc1394_camera_off(camera->handle, camera->id);
}


void
on_power_reset_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  int err;
  err=dc1394_init_camera(camera->handle, camera->id);
  

}

void
on_trigger_polarity_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  int err;
  err=dc1394_set_trigger_polarity(camera->handle,camera->id,togglebutton->active);
  feature_set->feature[FEATURE_TRIGGER].trigger_polarity=(int)togglebutton->active;
}


void
on_pan_op_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button, FEATURE_PAN);
}


void
on_pan_power_toggled                   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_PAN);
}


void
on_tilt_power_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_TILT);
}


void
on_tilt_op_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_TILT);
}


void
on_zoom_power_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_ZOOM);
}


void
on_zoom_op_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_ZOOM);
}


void
on_focus_op_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_FOCUS);
}


void
on_focus_power_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_FOCUS);
}


void
on_brightness_op_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_BRIGHTNESS);
}


void
on_brightness_power_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_BRIGHTNESS);
}


void
on_gamma_op_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_GAMMA);
}


void
on_gamma_power_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_GAMMA);
}


void
on_saturation_op_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_SATURATION);
}


void
on_saturation_power_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_SATURATION);
}


void
on_hue_op_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_HUE);
}


void
on_hue_power_toggled                   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_HUE);
}


void
on_whitebal_power_toggled              (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_WHITE_BALANCE);
}


void
on_whitebal_op_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_WHITE_BALANCE);
}


void
on_sharpness_op_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_SHARPNESS);
}


void
on_sharpness_power_toggled             (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_SHARPNESS);
}


void
on_exposure_power_toggled              (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_EXPOSURE);
}


void
on_exposure_op_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_EXPOSURE);
}


void
on_iris_power_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_IRIS);
}


void
on_iris_op_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_IRIS);
}


void
on_shutter_power_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_SHUTTER);
}


void
on_shutter_op_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_SHUTTER);
}


void
on_gain_power_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_GAIN);
}


void
on_gain_op_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_GAIN);
}


void
on_filter_power_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_OPTICAL_FILTER);
}


void
on_filter_op_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_OPTICAL_FILTER);
}


void
on_pan_man_toggled                     (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_PAN);
}


void
on_tilt_man_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_TILT);
}


void
on_zoom_man_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_ZOOM);
}


void
on_focus_man_toggled                   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_FOCUS);
}


void
on_brightness_man_toggled              (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_BRIGHTNESS);
}


void
on_gamma_man_toggled                   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_GAMMA);
}


void
on_saturation_man_toggled              (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_SATURATION);
}


void
on_hue_man_toggled                     (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_HUE);
}


void
on_whitebal_man_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_WHITE_BALANCE);
}


void
on_sharpness_man_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_SHARPNESS);
}


void
on_exposure_man_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_EXPOSURE);
}


void
on_iris_man_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_IRIS);
}


void
on_shutter_man_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_SHUTTER);
}


void
on_gain_man_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_GAIN);
}


void
on_filter_man_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_OPTICAL_FILTER);
}


void
on_trigger_mode_activate              (GtkMenuItem     *menuitem,
				       gpointer         user_data)
{
  int err;
  err=dc1394_set_trigger_mode(camera->handle, camera->id, (int)user_data);
  feature_set->feature[FEATURE_TRIGGER].trigger_mode=(int)user_data;
}

void
on_trigger_external_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  int err;
  err=dc1394_feature_on_off(camera->handle, camera->id, FEATURE_TRIGGER, togglebutton->active);
  feature_set->feature[FEATURE_TRIGGER].is_on=togglebutton->active;
  gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET (togglebutton),"fps"),!togglebutton->active);
  gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET (togglebutton),"trigger_mode"),togglebutton->active);
  gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET (togglebutton),"trigger_polarity"),
			   togglebutton->active &&
			   feature_set->feature[FEATURE_TRIGGER].trigger_polarity);
}

void
on_memory_channel_activate              (GtkMenuItem     *menuitem,
					 gpointer         user_data)
{
  misc_info->save_channel=(int)user_data; // user data is an int.
  misc_info->load_channel=(int)user_data; // user data is an int.
  // do not allow save on factory defaults:
  gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET (menuitem),"save_mem"),(misc_info->mem_channel_number>0)&&((int)user_data>0));
}


void
on_status_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (status_window);
}


void
on_capture_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (capture_window);
}


void
on_capqual_man_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_CAPTURE_QUALITY);
}


void
on_capqual_op_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_CAPTURE_QUALITY);
}


void
on_capqual_power_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_CAPTURE_QUALITY);
}


void
on_capsize_man_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_CAPTURE_SIZE);
}


void
on_capsize_op_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_CAPTURE_SIZE);
}


void
on_capsize_power_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_CAPTURE_SIZE);
}

void
on_capture_start_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  // TODO:
  // use video1394 to capture pictures:
}


void
on_capture_stop_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  // TODO:
  // release video1394 ressources, write file
}


void
on_capture_single_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  // TODO:
  // get one frame and leave
}

void
on_temp_power_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_TEMPERATURE);
}


void
on_temp_man_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_TEMPERATURE);
}


void
on_temp_op_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_TEMPERATURE);
}


void
on_temp_window_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (temperature_window);
}


void
on_set_all_auto_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  // TODO:
  // for each feature, set to auto if available, else set to OP/auto, else do nothing
}


void
on_set_all_man_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  // TODO:
  // for each feature, set to man if available, else do nothing
}


void
on_lock_setup_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  // TODO:
  // pressed: set all to man, then lock all controls
  // depressed: unlock all available controls
}

// the two following funcs are under development...

void
on_load_mem_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  int err, temp;
  err=dc1394_memory_load(camera->handle,camera->id, misc_info->load_channel);
  err=dc1394_get_memory_load_ch(camera->handle,camera->id, &temp);
  // UPDATE ALL WIDGETS HERE

}

void
on_save_mem_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{ 
  int err, timeout;
  dc1394bool_t boolean;
  err=dc1394_set_memory_save_ch(camera->handle,camera->id, misc_info->save_channel);
  err=dc1394_memory_save(camera->handle,camera->id);
  err=dc1394_get_memory_save_ch(camera->handle,camera->id,&timeout);
  boolean=TRUE;
  timeout=LOOP_RETRIES;
  while(boolean & (timeout>0))
    { // wait for save to be completed:
      usleep(LOOP_SLEEP);
      err=dc1394_is_memory_save_in_operation(camera->handle,camera->id, &boolean);
      timeout--;
    }
  
}


void
on_format7_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (format7_window);
}


void
on_iso_start_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  int err, channel, speed;
  // future addition: if no iso setup made, do it first.
  err=dc1394_get_iso_channel_and_speed(camera->handle, camera->id, &channel, &speed); //??
  err=dc1394_start_iso_transmission(camera->handle,camera->id);
  misc_info->is_iso_on=DC1394_TRUE;
  //printf("err = %d\n",err);
}


void
on_iso_stop_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  int err;
  //printf("Stopping...\n");
  err=dc1394_stop_iso_transmission(camera->handle,camera->id);
  misc_info->is_iso_on=DC1394_FALSE;
  //printf("... Stopped: err= %d\n",err);
}


void
on_iso_restart_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  int err, channel, speed;
  err=dc1394_stop_iso_transmission(camera->handle,camera->id);
  // future addition: if no iso setup made, do it first.
  err=dc1394_get_iso_channel_and_speed(camera->handle, camera->id, &channel, &speed);//??
  err=dc1394_start_iso_transmission(camera->handle,camera->id);
  misc_info->is_iso_on=DC1394_TRUE;
}


void
on_camera_select_activate              (GtkMenuItem     *menuitem,
					 gpointer         user_data)
{
  // set current camera pointers:
  current_camera=(int)user_data;
  camera=&cameras[current_camera];
  feature_set=&feature_sets[current_camera];
  misc_info=&misc_infos[current_camera];

  // redraw all:
  ActivateCapabilities();
}
