/*
 * Copyright (C) 2000-2001 Damien Douxchamps  <douxchamps@ieee.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "callback_proc.h"
#include "update.h"
#include <libdc1394/dc1394_control.h>
#include <libraw1394/raw1394.h>
#include "video1394.h"

extern GtkWidget *commander_window;
extern GtkWidget *porthole_window;
extern GtkWidget *color_window;
extern GtkWidget *status_window;
extern GtkWidget *aperture_window;
extern GtkWidget *configure_window;
extern GtkWidget *capture_window;
extern GtkWidget *temperature_window;
extern dc1394_camerainfo *camera;
extern dc1394_feature_set *feature_set;
extern dc1394_miscinfo *misc_info;
extern char* feature_op_list[NUM_FEATURES];
extern char* feature_list[NUM_FEATURES];
extern char* feature_frame_list[NUM_FEATURES];
extern char* feature_scale_list[NUM_FEATURES];
extern char* feature_man_list[NUM_FEATURES];
extern char* feature_auto_list[NUM_FEATURES];
extern char* feature_power_list[NUM_FEATURES];
extern char* fps_label_list[6];


void UpdateRange(GtkWidget* current_window, int feature)
{
  dc1394bool_t power_on, auto_mode;

  power_on=feature_set->feature[feature].is_on;
  // the power is on if:
  // 1.- the power is on (obvious!)
  // 2.- the feature cannot be powered on/off but is available (in which case it is supposed to be on)
  power_on=power_on || (!feature_set->feature[feature].on_off_capable && feature_set->feature[feature].available);
  auto_mode=feature_set->feature[feature].auto_active;

  // grab&set power status:
  gtk_toggle_button_set_active((GtkToggleButton*)lookup_widget(current_window, feature_power_list[feature]),
			       power_on );

  // grab&set auto/man status, plus sensitive range in man mode:
  gtk_toggle_button_set_active((GtkToggleButton*)lookup_widget(current_window, feature_auto_list[feature]),
			       power_on && auto_mode);
  gtk_toggle_button_set_active((GtkToggleButton*)lookup_widget(current_window, feature_man_list[feature]),
			       power_on && !auto_mode);

  switch(feature)
    {
      case FEATURE_WHITE_BALANCE:
	gtk_widget_set_sensitive(lookup_widget(current_window, "whitebal_BU_scale"),(!auto_mode) && power_on);
	gtk_widget_set_sensitive(lookup_widget(current_window, "whitebal_RV_scale"),(!auto_mode) && power_on);break;
      case FEATURE_TEMPERATURE:
	// the only changeable range is the target one, the other is just an indicator.
	gtk_widget_set_sensitive(lookup_widget(current_window, "temp_goal_scale"),(!auto_mode) && power_on);break;
      default:
	gtk_widget_set_sensitive(lookup_widget(current_window, feature_scale_list[feature]), (!auto_mode) && power_on);
    }

  // grab&set range value if readable:
  UpdateRangeValue(current_window,feature);
}

void UpdateRangeValue(GtkWidget* widget, int feature)
{
  // NOTE: range values are the only value the camera changes by itself. Contrary to other items
  //       (like auto/man or power), this item cam thus change without the user's action. It is
  //       therefor mendatory to read the CAMERA value and not the value present in "feature_set".
  //       Moreover, we must WRITE the value read in the "feature_set" 'value' field.

  int  err, value, valueBU, valueRV, valuegoal, valuecurrent;
  GtkAdjustment* adj;

  // grab&set range value if readable:
  if (feature_set->feature[feature].readout_capable)
    { switch(feature)
      {
        case FEATURE_WHITE_BALANCE:
	  err=dc1394_get_white_balance(camera->handle,camera->id,&valueBU,&valueRV);
	  adj=gtk_range_get_adjustment(GTK_RANGE (lookup_widget(widget, "whitebal_BU_scale")));
	  gtk_adjustment_set_value(adj, valueBU);
	  feature_set->feature[feature].BU_value=valueBU;
	  //printf("valueBU: %d\n",valueBU);
	  adj=gtk_range_get_adjustment(GTK_RANGE (lookup_widget(widget, "whitebal_RV_scale")));
	  gtk_adjustment_set_value(adj, valueRV);
	  //printf("valueRV: %d\n",valueRV);
	  feature_set->feature[feature].RV_value=valueRV;break;
        case FEATURE_TEMPERATURE:
          err=dc1394_get_temperature(camera->handle,camera->id,&valuegoal,&valuecurrent);
	  adj=gtk_range_get_adjustment(GTK_RANGE (lookup_widget(widget, "temp_goal_scale")));
	  gtk_adjustment_set_value(adj, valuegoal);
	  feature_set->feature[feature].target_value=valuegoal;
	  adj=gtk_range_get_adjustment(GTK_RANGE (lookup_widget(widget, "temp_current_scale")));
	  gtk_adjustment_set_value(adj, value);
	  feature_set->feature[feature].value=value;break;
        default:
	  err=dc1394_get_feature_value(camera->handle,camera->id,feature,&value);
	  adj=gtk_range_get_adjustment(GTK_RANGE (lookup_widget(widget, feature_scale_list[feature])));
	  gtk_adjustment_set_value(adj, value);
	  //printf("value: %d\n",value);
	  feature_set->feature[feature].value=value;
      }
    }

}

void
UpdateFpsMenu(void)
{
  int i, err;
  //mode, format;
  quadlet_t value;
  GtkWidget* fps;
  GtkWidget* fps_menu;
  GtkWidget* glade_menuitem;
  int index[NUM_FRAMERATES];
  int k=0;
  int new_framerate=0;
  dc1394bool_t cont=DC1394_TRUE;

  // select current format and set available framerates:

  //err=dc1394_get_video_mode(camera->handle, camera->id, &mode);
  //err=dc1394_get_video_format(camera->handle, camera->id, &format);
  //err=dc1394_get_video_framerate(camera->handle, camera->id, &k);
  err=dc1394_query_supported_framerates(camera->handle, camera->id, misc_info->format, misc_info->mode, &value);
  //printf("fps reg value: %x for mode %d and format %d with fps=%d\n",value, misc_info->mode,misc_info->format,k );

  gtk_widget_destroy(GTK_WIDGET (lookup_widget(commander_window,"fps"))); // remove previous menu

  fps = gtk_option_menu_new ();
  gtk_widget_ref (fps);
  gtk_object_set_data_full (GTK_OBJECT (commander_window), "fps", fps,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fps);
  gtk_table_attach_defaults (GTK_TABLE (lookup_widget(commander_window,"table17")), fps, 0, 2, 2, 3);
  gtk_container_set_border_width (GTK_CONTAINER (fps), 1);

  fps_menu = gtk_menu_new ();

  k=0;
  for (i=0;i<NUM_FRAMERATES;i++)
    {
      if  ( value & (0x1<< (31-i) ) ) //31 to 31-num_framerates 
	{
	  index[i]=k;
	  k++;
	  glade_menuitem = gtk_menu_item_new_with_label (_(fps_label_list[i]));
	  gtk_widget_show (glade_menuitem);
	  gtk_menu_append (GTK_MENU (fps_menu), glade_menuitem);
	  gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			      GTK_SIGNAL_FUNC (on_fps_activate),
			      (int*)i);
	}
      else
	index[i]=-1; // framerate not supported
    }
  gtk_option_menu_set_menu (GTK_OPTION_MENU (fps), fps_menu);

  // here we set the sensitiveness, AFTER the 'gtk_option_menu_set_menu' command:
  gtk_widget_set_sensitive (lookup_widget(commander_window,"fps"),
 			    !(GTK_TOGGLE_BUTTON (lookup_widget(commander_window,"trigger_external")))->active);

  // sets the active menu item:
  if (index[misc_info->framerate]<0) // previously selected framerate unsupported!!
    { // automatically switch to nearest fps available
      for (i=1;i<=((NUM_FRAMERATES>>1)+1);i++) // search radius is num_framerates/2. +1 for safe rounding
	{ 
	  if (((misc_info->framerate-i)>=0) && cont)
	    {
	      if (index[misc_info->framerate-i]>=0) // try down
		{ 
		  new_framerate=misc_info->framerate-i;
		  cont=DC1394_FALSE;
		}
	    }
	  if (((misc_info->framerate+i)<NUM_FRAMERATES) && cont)
	    {
	      if (index[misc_info->framerate+i]>=0) // try up
		{ 
		  new_framerate=misc_info->framerate+i;
		  cont=DC1394_FALSE;
		}
	    }
	}
      printf("Framerate not valid. Updating to nearest: %s\n",fps_label_list[new_framerate]);
      IsoFlowCheck();
      err=dc1394_set_video_framerate(camera->handle,camera->id,new_framerate);
      IsoFlowResume();
      misc_info->framerate=new_framerate;
    }
  gtk_option_menu_set_history (GTK_OPTION_MENU (fps), index[misc_info->framerate]);

}

